/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* vim:set ts=2 sw=2 sts=2 et cindent: */
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

#include "nsIDOMEvent.idl"

/**
 * ProgressEvent can be used for measuring progress.
 */

[scriptable, builtinclass, uuid(e0682338-4c3f-4d3a-9487-d7492ea76335)]
interface nsIDOMProgressEvent : nsIDOMEvent
{
  readonly attribute boolean lengthComputable;
  readonly attribute unsigned long long loaded;
  readonly attribute unsigned long long total;
  [noscript]
  void initProgressEvent(in DOMString typeArg,
                         in boolean canBubbleArg,
                         in boolean cancelableArg,
                         in boolean lengthComputableArg,
                         in unsigned long long loadedArg,
                         in unsigned long long totalArg);
};

dictionary ProgressEventInit : EventInit {
  boolean lengthComputable;
  unsigned long long loaded;
  unsigned long long total;
};
